@@ -1,9 +1,10 @@
-import numpy as np
-import pandas as pd
 import json
 from abc import ABC, abstractmethod
 from typing import Callable, List
 
+import numpy as np
+import pandas as pd
+
 
 class Epsilon(ABC):
     """
@@ -69,12 +70,14 @@ class Epsilon(ABC):
             The sampler used in ABCSMC.
         """
 
-    def update(self,
-               t: int,
-               get_weighted_distances: Callable[[], pd.DataFrame],
-               get_all_records: Callable[[], List[dict]],
-               acceptance_rate: float,
-               acceptor_config: dict):
+    def update(
+        self,
+        t: int,
+        get_weighted_distances: Callable[[], pd.DataFrame],
+        get_all_records: Callable[[], List[dict]],
+        acceptance_rate: float,
+        acceptor_config: dict,
+    ):
         """
         Update epsilon value to be used as acceptance criterion for
         generation t.
@@ -104,8 +107,7 @@ class Epsilon(ABC):
         pass
 
     @abstractmethod
-    def __call__(self,
-                 t: int) -> float:
+    def __call__(self, t: int) -> float:
         """
         Get epsilon value for generation t.
 
@@ -174,6 +176,5 @@ class NoEpsilon(Epsilon):
     def __init__(self):
         super().__init__()
 
-    def __call__(self,
-                 t: int) -> float:
+    def __call__(self, t: int) -> float:
         return np.nan
