@@ -1,31 +1,49 @@
 """Redis based sampler base class and dynamic scheduling samplers."""
 
-import numpy as np
-from time import sleep
-from datetime import datetime
-import cloudpickle as pickle
 import copy
 import logging
-from redis import StrictRedis
+from datetime import datetime
+from time import sleep
 from typing import Callable, Dict, List, Tuple
-from jabbar import jabbar
 
+import cloudpickle as pickle
+import numpy as np
+from jabbar import jabbar
+from redis import StrictRedis
+
+from ...acceptor import Acceptor
+from ...distance import Distance
+from ...epsilon import Epsilon
 from ...inference_util import (
     AnalysisVars,
     create_simulate_function,
     evaluate_preliminary_particle,
     termination_criteria_fulfilled,
 )
-from ...distance import Distance
-from ...epsilon import Epsilon
-from ...acceptor import Acceptor
+from ...population import Sample
 from ...sampler import Sampler
 from ...weighted_statistics import effective_sample_size
-from ...population import Sample
 from .cmd import (
-    SSA, N_EVAL, N_ACC, N_REQ, N_FAIL, N_LOOKAHEAD_EVAL, ALL_ACCEPTED,
-    N_WORKER, QUEUE, MSG, START, MODE, DYNAMIC, SLEEP_TIME, BATCH_SIZE,
-    IS_LOOK_AHEAD, ANALYSIS_ID, GENERATION, MAX_N_EVAL_LOOK_AHEAD, DONE_IXS,
+    ALL_ACCEPTED,
+    ANALYSIS_ID,
+    BATCH_SIZE,
+    DONE_IXS,
+    DYNAMIC,
+    GENERATION,
+    IS_LOOK_AHEAD,
+    MAX_N_EVAL_LOOK_AHEAD,
+    MODE,
+    MSG,
+    N_ACC,
+    N_EVAL,
+    N_FAIL,
+    N_LOOKAHEAD_EVAL,
+    N_REQ,
+    N_WORKER,
+    QUEUE,
+    SLEEP_TIME,
+    SSA,
+    START,
     idfy,
 )
 from .redis_logging import RedisSamplerLogger
@@ -60,11 +78,11 @@ class RedisSamplerBase(Sampler):
         log_file: str = None,
     ):
         super().__init__()
-        logger.debug(
-            f"Redis sampler: host={host} port={port}")
+        logger.debug(f"Redis sampler: host={host} port={port}")
         # handles the connection to the redis-server
         self.redis: StrictRedis = StrictRedis(
-            host=host, port=port, password=password)
+            host=host, port=port, password=password
+        )
         self.logger = RedisSamplerLogger(log_file)
 
     def n_worker(self) -> int:
@@ -83,7 +101,8 @@ class RedisSamplerBase(Sampler):
         super().set_analysis_id(analysis_id)
         if self.redis.get(ANALYSIS_ID):
             raise AssertionError(
-                "The server seems busy with an analysis already")
+                "The server seems busy with an analysis already"
+            )
         self.redis.set(ANALYSIS_ID, analysis_id)
 
     def sample_until_n_accepted(
@@ -189,12 +208,13 @@ class RedisEvalParallelSampler(RedisSamplerBase):
         batch_size: int = 1,
         look_ahead: bool = False,
         look_ahead_delay_evaluation: bool = True,
-        max_n_eval_look_ahead_factor: float = 10.,
+        max_n_eval_look_ahead_factor: float = 10.0,
         wait_for_all_samples: bool = False,
         log_file: str = None,
     ):
         super().__init__(
-            host=host, port=port, password=password, log_file=log_file)
+            host=host, port=port, password=password, log_file=log_file
+        )
         self.batch_size: int = batch_size
         self.look_ahead: bool = look_ahead
         self.look_ahead_delay_evaluation: bool = look_ahead_delay_evaluation
@@ -202,8 +222,15 @@ class RedisEvalParallelSampler(RedisSamplerBase):
         self.wait_for_all_samples: bool = wait_for_all_samples
 
     def sample_until_n_accepted(
-            self, n, simulate_one, t, *,
-            max_eval=np.inf, all_accepted=False, ana_vars=None) -> Sample:
+        self,
+        n,
+        simulate_one,
+        t,
+        *,
+        max_eval=np.inf,
+        all_accepted=False,
+        ana_vars=None,
+    ) -> Sample:
         # get the analysis id
         ana_id = self.analysis_id
 
@@ -215,7 +242,8 @@ class RedisEvalParallelSampler(RedisSamplerBase):
             # update the SSA function
             self.redis.set(
                 idfy(SSA, ana_id, t),
-                pickle.dumps((simulate_one, self.sample_factory)))
+                pickle.dumps((simulate_one, self.sample_factory)),
+            )
             # update the required population size
             self.redis.set(idfy(N_REQ, ana_id, t), n)
             # let the workers know they should update their ssa
@@ -228,8 +256,12 @@ class RedisEvalParallelSampler(RedisSamplerBase):
         else:
             # set up all variables for the new generation
             self.start_generation_t(
-                n=n, t=t, simulate_one=simulate_one, all_accepted=all_accepted,
-                is_look_ahead=False)
+                n=n,
+                t=t,
+                simulate_one=simulate_one,
+                all_accepted=all_accepted,
+                is_look_ahead=False,
+            )
 
         # for the results
         id_results = []
@@ -245,10 +277,14 @@ class RedisEvalParallelSampler(RedisSamplerBase):
                 sample_with_id = pickle.loads(dump)
 
                 # check whether the sample is really acceptable
-                sample_with_id, any_particle_accepted = \
-                    post_check_acceptance(
-                        sample_with_id, ana_id=ana_id, t=t, redis=self.redis,
-                        ana_vars=ana_vars, logger=self.logger)
+                sample_with_id, any_particle_accepted = post_check_acceptance(
+                    sample_with_id,
+                    ana_id=ana_id,
+                    t=t,
+                    redis=self.redis,
+                    ana_vars=ana_vars,
+                    logger=self.logger,
+                )
 
                 if any_particle_accepted:
                     # append to collected results
@@ -257,8 +293,12 @@ class RedisEvalParallelSampler(RedisSamplerBase):
 
         # maybe head-start the next generation already
         self.maybe_start_next_generation(
-            t=t, n=n, id_results=id_results, all_accepted=all_accepted,
-            ana_vars=ana_vars)
+            t=t,
+            n=n,
+            id_results=id_results,
+            all_accepted=all_accepted,
+            ana_vars=ana_vars,
+        )
 
         # wait until all relevant simulations done
         if self.wait_for_all_samples:
@@ -268,9 +308,9 @@ class RedisEvalParallelSampler(RedisSamplerBase):
             # we only need to wait for simulations that were started
             #  before the last started one among the first n accepted ones
             #  as later once would be discarded anyway
-            max_ix = sorted(id_result[0] for id_result in id_results)[n-1]
+            max_ix = sorted(id_result[0] for id_result in id_results)[n - 1]
             # first time index is 1
-            missing_ixs = set(range(1, max_ix+1))
+            missing_ixs = set(range(1, max_ix + 1))
             while (
                 # check whether any active evaluation was started earlier
                 missing_ixs
@@ -302,10 +342,14 @@ class RedisEvalParallelSampler(RedisSamplerBase):
             sample_with_id = pickle.loads(dump)
 
             # check whether the sample is really acceptable
-            sample_with_id, any_particle_accepted = \
-                post_check_acceptance(
-                    sample_with_id, ana_id=ana_id, t=t, redis=self.redis,
-                    ana_vars=ana_vars, logger=self.logger)
+            sample_with_id, any_particle_accepted = post_check_acceptance(
+                sample_with_id,
+                ana_id=ana_id,
+                t=t,
+                redis=self.redis,
+                ana_vars=ana_vars,
+                logger=self.logger,
+            )
 
             if any_particle_accepted:
                 # append to collected results
@@ -320,7 +364,7 @@ class RedisEvalParallelSampler(RedisSamplerBase):
         if self.wait_for_all_samples:
             self.clear_generation_t(t=t)
         else:
-            for _t in range(-1, t+1):
+            for _t in range(-1, t + 1):
                 n_worker_b = self.redis.get(idfy(N_WORKER, ana_id, _t))
                 if n_worker_b is not None and int(n_worker_b.decode()) == 0:
                     # TODO For fast-running models, communication does not
@@ -334,8 +378,8 @@ class RedisEvalParallelSampler(RedisSamplerBase):
 
         # logging
         self.logger.add_row(
-            t=t, n_evaluated=self.nr_evaluations_,
-            n_lookahead=n_lookahead_eval)
+            t=t, n_evaluated=self.nr_evaluations_, n_lookahead=n_lookahead_eval
+        )
         self.logger.write()
 
         # weight sub-populations suitably
@@ -356,27 +400,31 @@ class RedisEvalParallelSampler(RedisSamplerBase):
         ana_id = self.analysis_id
 
         # write initial values to pipeline
-        (self.redis.pipeline()
-         # initialize variables for time t
-         .set(idfy(SSA, ana_id, t),
-              pickle.dumps((simulate_one, self.sample_factory)))
-         .set(idfy(N_EVAL, ana_id, t), 0)
-         .set(idfy(N_ACC, ana_id, t), 0)
-         .set(idfy(N_REQ, ana_id, t), n)
-         .set(idfy(N_FAIL, ana_id, t), 0)
-         .set(idfy(N_LOOKAHEAD_EVAL, ana_id, t), 0)
-         # encode as int
-         .set(idfy(ALL_ACCEPTED, ana_id, t), int(all_accepted))
-         .set(idfy(N_WORKER, ana_id, t), 0)
-         .set(idfy(BATCH_SIZE, ana_id, t), self.batch_size)
-         # encode as int
-         .set(idfy(IS_LOOK_AHEAD, ana_id, t), int(is_look_ahead))
-         .set(idfy(MAX_N_EVAL_LOOK_AHEAD, ana_id, t), max_n_eval_look_ahead)
-         .set(idfy(MODE, ana_id, t), DYNAMIC)
-         # update the current-generation variable
-         .set(idfy(GENERATION, ana_id), t)
-         # execute all commands
-         .execute())
+        (
+            self.redis.pipeline()
+            # initialize variables for time t
+            .set(
+                idfy(SSA, ana_id, t),
+                pickle.dumps((simulate_one, self.sample_factory)),
+            )
+            .set(idfy(N_EVAL, ana_id, t), 0)
+            .set(idfy(N_ACC, ana_id, t), 0)
+            .set(idfy(N_REQ, ana_id, t), n)
+            .set(idfy(N_FAIL, ana_id, t), 0)
+            .set(idfy(N_LOOKAHEAD_EVAL, ana_id, t), 0)
+            # encode as int
+            .set(idfy(ALL_ACCEPTED, ana_id, t), int(all_accepted))
+            .set(idfy(N_WORKER, ana_id, t), 0)
+            .set(idfy(BATCH_SIZE, ana_id, t), self.batch_size)
+            # encode as int
+            .set(idfy(IS_LOOK_AHEAD, ana_id, t), int(is_look_ahead))
+            .set(idfy(MAX_N_EVAL_LOOK_AHEAD, ana_id, t), max_n_eval_look_ahead)
+            .set(idfy(MODE, ana_id, t), DYNAMIC)
+            # update the current-generation variable
+            .set(idfy(GENERATION, ana_id), t)
+            # execute all commands
+            .execute()
+        )
 
         # publish start message
         self.redis.publish(MSG, START)
@@ -400,22 +448,24 @@ class RedisEvalParallelSampler(RedisSamplerBase):
         """
         ana_id = self.analysis_id
         # delete keys from pipeline
-        (self.redis.pipeline()
-         .delete(idfy(SSA, ana_id, t))
-         .delete(idfy(N_EVAL, ana_id, t))
-         .delete(idfy(N_ACC, ana_id, t))
-         .delete(idfy(N_REQ, ana_id, t))
-         .delete(idfy(N_FAIL, ana_id, t))
-         .delete(idfy(N_LOOKAHEAD_EVAL, ana_id, t))
-         .delete(idfy(ALL_ACCEPTED, ana_id, t))
-         .delete(idfy(N_WORKER, ana_id, t))
-         .delete(idfy(BATCH_SIZE, ana_id, t))
-         .delete(idfy(IS_LOOK_AHEAD, ana_id, t))
-         .delete(idfy(MAX_N_EVAL_LOOK_AHEAD, ana_id, t))
-         .delete(idfy(MODE, ana_id, t))
-         .delete(idfy(DONE_IXS, ana_id, t))
-         .delete(idfy(QUEUE, ana_id, t))
-         .execute())
+        (
+            self.redis.pipeline()
+            .delete(idfy(SSA, ana_id, t))
+            .delete(idfy(N_EVAL, ana_id, t))
+            .delete(idfy(N_ACC, ana_id, t))
+            .delete(idfy(N_REQ, ana_id, t))
+            .delete(idfy(N_FAIL, ana_id, t))
+            .delete(idfy(N_LOOKAHEAD_EVAL, ana_id, t))
+            .delete(idfy(ALL_ACCEPTED, ana_id, t))
+            .delete(idfy(N_WORKER, ana_id, t))
+            .delete(idfy(BATCH_SIZE, ana_id, t))
+            .delete(idfy(IS_LOOK_AHEAD, ana_id, t))
+            .delete(idfy(MAX_N_EVAL_LOOK_AHEAD, ana_id, t))
+            .delete(idfy(MODE, ana_id, t))
+            .delete(idfy(DONE_IXS, ana_id, t))
+            .delete(idfy(QUEUE, ana_id, t))
+            .execute()
+        )
 
     def maybe_start_next_generation(
         self,
@@ -460,21 +510,22 @@ class RedisEvalParallelSampler(RedisSamplerBase):
 
         # acceptance rate
         nr_evaluations = int(
-            self.redis.get(idfy(N_EVAL, self.analysis_id, t)).decode())
+            self.redis.get(idfy(N_EVAL, self.analysis_id, t)).decode()
+        )
         acceptance_rate = len(population) / nr_evaluations
 
         # check if any termination criterion (based on the current data)
         #  is likely to be fulfilled after the current generation
 
-        total_nr_simulations = \
+        total_nr_simulations = (
             ana_vars.prev_total_nr_simulations + nr_evaluations
+        )
         walltime = datetime.now() - ana_vars.init_walltime
 
         if termination_criteria_fulfilled(
             current_eps=ana_vars.eps(t),
             min_eps=ana_vars.min_eps,
-            stop_if_single_model_alive=  # noqa: E251
-            ana_vars.stop_if_single_model_alive,
+            stop_if_single_model_alive=ana_vars.stop_if_single_model_alive,  # noqa: E251
             nr_of_models_alive=population.nr_of_models_alive(),
             acceptance_rate=acceptance_rate,
             min_acceptance_rate=ana_vars.min_acceptance_rate,
@@ -482,24 +533,29 @@ class RedisEvalParallelSampler(RedisSamplerBase):
             max_total_nr_simulations=ana_vars.max_total_nr_simulations,
             walltime=walltime,
             max_walltime=ana_vars.max_walltime,
-            t=t, max_t=ana_vars.max_t,
+            t=t,
+            max_t=ana_vars.max_t,
         ):
             # do not head-start a new generation as this is likely not needed
             return
 
         # create a preliminary simulate_one function
         simulate_one_prel = create_preliminary_simulate_one(
-            t=t + 1, population=population,
+            t=t + 1,
+            population=population,
             delay_evaluation=self.look_ahead_delay_evaluation,
-            ana_vars=ana_vars)
+            ana_vars=ana_vars,
+        )
 
         # maximum number of look-ahead evaluations
         if self.look_ahead_delay_evaluation:
             # set maximum evaluations to previous simulations * const
             nr_evaluations_ = int(
-                self.redis.get(idfy(N_EVAL, self.analysis_id, t)).decode())
-            max_n_eval_look_ahead = \
+                self.redis.get(idfy(N_EVAL, self.analysis_id, t)).decode()
+            )
+            max_n_eval_look_ahead = (
                 nr_evaluations_ * self.max_n_eval_look_ahead_factor
+            )
         else:
             # no maximum necessary as samples are directly evaluated
             max_n_eval_look_ahead = np.inf
@@ -507,9 +563,13 @@ class RedisEvalParallelSampler(RedisSamplerBase):
         # head-start the next generation
         #  all_accepted is most certainly False for t>0
         self.start_generation_t(
-            n=n, t=t + 1, simulate_one=simulate_one_prel,
-            all_accepted=False, is_look_ahead=True,
-            max_n_eval_look_ahead=max_n_eval_look_ahead)
+            n=n,
+            t=t + 1,
+            simulate_one=simulate_one_prel,
+            all_accepted=False,
+            is_look_ahead=True,
+            max_n_eval_look_ahead=max_n_eval_look_ahead,
+        )
 
     def create_sample(self, id_results: List[Tuple], n: int) -> Sample:
         """Create a single sample result.
@@ -537,7 +597,7 @@ class RedisEvalParallelSampler(RedisSamplerBase):
         eps: Epsilon,
         acceptor: Acceptor,
     ) -> None:
-        """"Check analysis variables appropriateness for sampling."""
+        """ "Check analysis variables appropriateness for sampling."""
         if self.look_ahead_delay_evaluation:
             # nothing to be done
             return
@@ -550,7 +610,8 @@ class RedisEvalParallelSampler(RedisSamplerBase):
                 raise AssertionError(
                     f"{var.__class__.__name__} cannot be used in look-ahead "
                     "mode without delayed acceptance. Consider setting the "
-                    "sampler's `look_ahead_delay_evaluation` flag.")
+                    "sampler's `look_ahead_delay_evaluation` flag."
+                )
 
         check_bad(acceptor)
         check_bad(distance_function)
@@ -558,7 +619,10 @@ class RedisEvalParallelSampler(RedisSamplerBase):
 
 
 def create_preliminary_simulate_one(
-    t, population, delay_evaluation: bool, ana_vars: AnalysisVars,
+    t,
+    population,
+    delay_evaluation: bool,
+    ana_vars: AnalysisVars,
 ) -> Callable:
     """Create a preliminary simulate_one function for generation `t`.
 
@@ -590,19 +654,29 @@ def create_preliminary_simulate_one(
         transitions[m].fit(parameters, w)
 
     return create_simulate_function(
-        t=t, model_probabilities=model_probabilities,
+        t=t,
+        model_probabilities=model_probabilities,
         model_perturbation_kernel=ana_vars.model_perturbation_kernel,
-        transitions=transitions, model_prior=ana_vars.model_prior,
+        transitions=transitions,
+        model_prior=ana_vars.model_prior,
         parameter_priors=ana_vars.parameter_priors,
-        models=ana_vars.models, summary_statistics=ana_vars.summary_statistics,
-        x_0=ana_vars.x_0, distance_function=ana_vars.distance_function,
-        eps=ana_vars.eps, acceptor=ana_vars.acceptor,
-        evaluate=not delay_evaluation, proposal_id=-1,
+        models=ana_vars.models,
+        summary_statistics=ana_vars.summary_statistics,
+        x_0=ana_vars.x_0,
+        distance_function=ana_vars.distance_function,
+        eps=ana_vars.eps,
+        acceptor=ana_vars.acceptor,
+        evaluate=not delay_evaluation,
+        proposal_id=-1,
     )
 
 
 def post_check_acceptance(
-    sample_with_id, ana_id, t, redis, ana_vars,
+    sample_with_id,
+    ana_id,
+    t,
+    redis,
+    ana_vars,
     logger: RedisSamplerLogger,
 ) -> Tuple:
     """Check whether the sample is really acceptable.
@@ -627,7 +701,8 @@ def post_check_acceptance(
         if n_accepted != 1:
             # this should never happen
             raise AssertionError(
-                "Expected exactly one accepted particle in sample.")
+                "Expected exactly one accepted particle in sample."
+            )
 
         # increase general acceptance counter
         logger.n_accepted += 1
@@ -644,15 +719,15 @@ def post_check_acceptance(
         # this should never happen
         raise AssertionError(
             "Expected number of particles in sample: 1. "
-            f"Got: {len(sample.all_particles)}")
+            f"Got: {len(sample.all_particles)}"
+        )
 
     # from here on, we may assume that all particles (#=1) are yet to be judged
     logger.n_preliminary += 1
 
     # iterate over the 1 particle
     for particle in sample.all_particles:
-        particle = \
-            evaluate_preliminary_particle(particle, t, ana_vars)
+        particle = evaluate_preliminary_particle(particle, t, ana_vars)
 
         # react to acceptance
         if particle.accepted:
@@ -698,16 +773,21 @@ def self_normalize_within_subpopulations(sample: Sample, n: int) -> Sample:
 
     # get particles per proposal
     particles_per_prop = {
-        prop_id: [particle for particle in sample.accepted_particles
-                  if particle.proposal_id == prop_id]
-        for prop_id in prop_ids}
+        prop_id: [
+            particle
+            for particle in sample.accepted_particles
+            if particle.proposal_id == prop_id
+        ]
+        for prop_id in prop_ids
+    }
 
     # normalize weights by $ESS_l / sum_{i<=N_l} w^l_i$ for proposal id l
     # this is s.t. $sum_{i<=N_l} w^l_i \propto ESS_l$
     normalizations: Dict[int, float] = {}
     for prop_id, particles_for_prop in particles_per_prop.items():
         weights = np.array(
-            [particle.weight for particle in particles_for_prop])
+            [particle.weight for particle in particles_for_prop]
+        )
         ess = effective_sample_size(weights)
         total_weight = weights.sum()
         normalizations[prop_id] = ess / total_weight
@@ -719,6 +799,6 @@ def self_normalize_within_subpopulations(sample: Sample, n: int) -> Sample:
         else:
             # set weight of particles from populations None of which was
             #  accepted to 0 (until we start caring about those for real)
-            particle.weight = 0.
+            particle.weight = 0.0
 
     return sample
