@@ -12,13 +12,15 @@ These can be passed to :class:`pyabc.smc.ABCSMC` via the ``transitions``
 keyword argument.
 """
 
-from .base import Transition, DiscreteTransition, AggregatedTransition
-from .multivariatenormal import (MultivariateNormalTransition,
-                                 silverman_rule_of_thumb,
-                                 scott_rule_of_thumb)
+from .base import AggregatedTransition, DiscreteTransition, Transition
 from .exceptions import NotEnoughParticles
 from .grid_search import GridSearchCV
+from .jump import DiscreteJumpTransition, PerturbationKernel
 from .local_transition import LocalTransition
-from .randomwalk import DiscreteRandomWalkTransition
-from .jump import PerturbationKernel, DiscreteJumpTransition
 from .model import ModelPerturbationKernel
+from .multivariatenormal import (
+    MultivariateNormalTransition,
+    scott_rule_of_thumb,
+    silverman_rule_of_thumb,
+)
+from .randomwalk import DiscreteRandomWalkTransition
