@@ -1,9 +1,13 @@
-from pyabc.sampler import (MulticoreParticleParallelSampler,
-                           MulticoreEvalParallelSampler)
-from pyabc.population import Particle
-import pytest
 from multiprocessing import ProcessError
 
+import pytest
+
+from pyabc.population import Particle
+from pyabc.sampler import (
+    MulticoreEvalParallelSampler,
+    MulticoreParticleParallelSampler,
+)
+
 
 class UnpickleAble:
     def __init__(self):
@@ -13,15 +17,15 @@ class UnpickleAble:
         raise Exception
 
     def __call__(self, *args, **kwargs):
-        return Particle(
-            m=0, parameter={}, weight=1, sum_stat={}, distance=1)
+        return Particle(m=0, parameter={}, weight=1, sum_stat={}, distance=1)
 
 
 unpickleable = UnpickleAble()
 
 
-@pytest.fixture(params=[MulticoreParticleParallelSampler,
-                        MulticoreEvalParallelSampler])
+@pytest.fixture(
+    params=[MulticoreParticleParallelSampler, MulticoreEvalParallelSampler]
+)
 def sampler(request):
     return request.param()
 
@@ -31,8 +35,10 @@ def test_no_pickle(sampler):
 
 
 def raise_exception(*args):
-    raise Exception("Deliberate exception to be raised in the worker "
-                    "processes and to be propagated to the parent process.")
+    raise Exception(
+        "Deliberate exception to be raised in the worker "
+        "processes and to be propagated to the parent process."
+    )
 
 
 def test_exception_from_worker_propagated(sampler):
